#!/usr/bin/env python
#
# Transfer function script for suspensions electronics testing (SR785).  Modified from
# code by Yoichi Aso.
# cmreed 20100329
#
# TFSR785.py [-f filename] [-i ip_address] [-a gpib_address] [{-s|--startfreq} start_freq]
#                 [{-e|--stopfreq} stop_freq] [-n|--numpoints num_of_points] [--sweep Linear|Log] 
#                 [{-x|--excamp} excitation_amplitude] [{-c|--settlecycle} settle_cycle]
#                 [{-t|--intcycle} integration_cycle] [--ic1 AC|DC ] [--ic2 AC|DC ]
#                 [--ig1 GND|Float ] [--ig2 GND|Float ] [--armode UpOnly|Tracking ] [--title Title]
#                 [--memo Memo]
#                 

import re
import sys
import math
import optparse
import time
import pdb
import netgpib
import SR785
import termstatus
from pylab import *
import matplotlib.pyplot as plt


#Parse options
usage = """usage: %prog [options]

This program executes a transfer function measurement using SR785.
Various measurement conditions can be set using options.
The measurement results will be saved in FILENAME.txt.
"""

filetime = time.strftime("%Y-%m-%d.%H%M%S",time.gmtime()) # Date string for data file (default)


# Options below set for SUS transfer function measurements
parser = optparse.OptionParser(usage=usage)
parser.add_option("-f", "--file", dest="filename",
                  help="Output file name without an extension", default = filetime)
parser.add_option("-i", "--ip",
                  dest="ipAddress", default="169.254.0.1",
                  help="IP address/Host name")
parser.add_option("-a", "--address",
                  dest="gpibAddress", type="int", default=10,
                  help="GPIB device address")
parser.add_option("-s", "--startfreq",
                  dest="startFreq", default="10kHz",
                  help="Start frequency. Units can be mHz, Hz or kHz.")
parser.add_option("-e", "--stopfreq",
                  dest="stopFreq", default="100mHz",
                  help="Stop frequency. Units can be mHz, Hz or kHz.")
parser.add_option("-n", "--numpoints",
                  dest="numOfPoints", type="int", default="101",
                  help="Number of frequency points")
# For the number of points, using a +1 value (e.g. 101, 201) will allow
# the measured points to be exactly on orders of ten for a log measurement
# (e.g. 10Hz, 100Hz, 1kHz, etc.)
# DEA 14 Apr 2011
parser.add_option("--sweep",
                  dest="sweepType", type="string", default="Log",
                  help="Sweep type: Log or Linear (default: Log)")
parser.add_option("-x", "--excamp",
                  dest="excAmp", default="2500mV",
                  help="Excitation amplitude")
parser.add_option("-c", "--settlecycle",
                  dest="settleCycles", type="int", default="1",
                  help="Settle cycles")
parser.add_option("-t", "--intcycle",
                  dest="intCycles", type="int",default="10",
                  help="Integration cycles")
parser.add_option("--ic1",
                  dest="inputCoupling1", type="string",default="AC",
                  help="CH1 input coupling. DC or AC (default:AC)")
parser.add_option("--ic2",
                  dest="inputCoupling2", type="string",default="AC",
                  help="CH2 input coupling. DC or AC (default:AC)")
parser.add_option("--ig1",
                  dest="inputGND1", type="string",default="Float",
                  help="CH1 input grounding. Float or Ground (default:Float)")
parser.add_option("--ig2",
                  dest="inputGND2", type="string",default="Float",
                  help="CH2 input grounding. Float or Ground (default:Float)")
parser.add_option("--id1",
                  dest="inputDiff1", type="string",default="SE",
                  help="CH1 input Differential A-B, or single ended A")
parser.add_option("--id2",
                  dest="inputDiff2", type="string",default="SE",
                  help="CH1 input Differential A-B, or single ended A")
parser.add_option("--armode",
                  dest="arMode", type="string",default="UpOnly",
                  help="Auto range mode: UpOnly or Tracking (default: UpOnly)")
parser.add_option("--magcorrection",
                  dest="magcorrection", type="float",default="0.0",
                  help="Correction to apply to data in dB (formula: DATA - CORRECTION, default: 0.0)")
parser.add_option("--phasecorrection",
                  dest="phasecorrection", type="float",default="0.0",
                  help="Correction to apply to phase in deg (formula: DATA - CORRECTION, default: 0.0)")
parser.add_option("--curvefit",
                  dest="curvefit", type="string",default=False,
                  help="(Don't use.) Fit poles and zeroes? True or False(default).")
parser.add_option("--savecal",
                  dest="saveCal", default=False,
                  help="Will save this measurement into 785's trace 2 for calibration use.")
parser.add_option("--usecal",
                  dest="useCal", default=False,
                  help="Will use the saved trace 2 calibration measurement and subtract it from this measurement.")
parser.add_option("--title",
                  dest="title", type="string",default="",
                  help="Title of the measurement. The given string will be written into the parameter file.")
parser.add_option("--memo",
                  dest="memo", type="string",default="",
                  help="Use this option to note miscellaneous things.")


(options, args) = parser.parse_args()
# Create a netGPIB class object
print('Connecting to '+str(options.ipAddress)+' ...'),
gpibObj=netgpib.netGPIB(options.ipAddress, options.gpibAddress, '\004',0)
print('done.')

#File names
dataFileName=options.filename+'.txt'

print('Measurement data will be written into '+dataFileName)
print('Setting up parameters for the measurement...') 

#Prepare for the TF measurement

#Set output to GPIB
gpibObj.command("OUTX0")
time.sleep(0.1)

# Print IDN
print "Instrument ID: ",
idnString=gpibObj.query("*IDN?")
print idnString[:-1]
time.sleep(0.1)

#Input setup
if options.inputCoupling1 == "AC":
    icp1="1"
else:
    icp1="0"
gpibObj.command('I1CP'+icp1) #CH1 Input Coupling
time.sleep(0.1)

if options.inputCoupling2 == "AC":
    icp2="1"
else:
    icp2="0"
gpibObj.command('I2CP'+icp2) #CH2 Input Coupling
time.sleep(0.1)

if options.inputGND1 == "Float":
    igd1="0"
else:
    igd1="1"
gpibObj.command('I1GD'+igd1) #CH1 Input GND
time.sleep(0.1)

if options.inputGND2 == "Float":
    igd2="0"
else:
    igd2="1"
gpibObj.command('I2GD'+igd2) #CH2 Input GND
time.sleep(0.1)

if options.inputDiff1 == "SE":
    idf1="0"
else:
    idf1="1"
gpibObj.command('I1MD'+idf1) #CH1 Input A-B = 1; A = 0
time.sleep(0.1)

if options.inputDiff2 == "SE":
    idf1="0"
else:
    idf1="1"
gpibObj.command('I2MD'+idf1) #CH2 Input A-B = 1; A = 0
time.sleep(0.1)

gpibObj.command('A1RG0') #AutoRange Off
time.sleep(0.1)
gpibObj.command('A2RG0') #AutoRange Off
time.sleep(0.1)
if options.arMode == "Tracking":
    arModeID='1'
else:
    arModeID='0'
gpibObj.command('I1AR'+arModeID) #Auto Range Mode 
time.sleep(0.1)
gpibObj.command('I2AR'+arModeID) #Auto Range Mode 
time.sleep(0.1)
gpibObj.command('A1RG1') #AutoRange On
time.sleep(0.1)
gpibObj.command('A2RG1') #AutoRange On
time.sleep(0.1)

gpibObj.command('I1AF1') #Anti-Aliasing filter On
time.sleep(0.1)
gpibObj.command('I2AF1') #Anti-Aliasing filter On
time.sleep(0.1)

#Set measurement parameters
gpibObj.command('DFMT1') # Dual display
time.sleep(0.1)
gpibObj.command('ACTD0') # Active display 0
time.sleep(0.1)


gpibObj.command('PSDU0,0')  # PSD Unit Off - Must be done before changing
                            # measurement group.
time.sleep(0.1)
gpibObj.command('PSDU1,0')  # PSD Unit Off
time.sleep(0.1)
gpibObj.command('MGRP2,3') # Measurement Group = Swept Sine
time.sleep(0.1)
gpibObj.command('MEAS2,47') # Frequency Resp
time.sleep(0.1)
gpibObj.command('VIEW0,0') # Disp 0 = LogMag
time.sleep(0.1)
gpibObj.command('VIEW1,6') # Dsip 1 = Unwrapped Phase (std Phase ='5')
time.sleep(0.1)
gpibObj.command('UNDB0,1') # dB ON
time.sleep(0.1)
gpibObj.command('UNPK0,0') # PK Unit Off
time.sleep(0.1)
gpibObj.command('UNDB1,0') # dB OFF
time.sleep(0.1)
gpibObj.command('UNPK1,0') # PK Unit Off
time.sleep(0.1)
gpibObj.command('UNPH1,0') # Phase Unit deg.
time.sleep(0.1)
gpibObj.command('DISP0,1') # Live display on
time.sleep(0.1)
gpibObj.command('DISP1,1') # Live display on
time.sleep(0.1)
gpibObj.command('SSCY2,'+str(options.settleCycles)) # Settle cycles
time.sleep(0.1)
gpibObj.command('SICY2,'+str(options.intCycles)) # Integration cycles
time.sleep(0.1)
gpibObj.command('SSTR2,'+options.startFreq) #Start frequency
time.sleep(0.1)
gpibObj.command('SSTP2,'+options.stopFreq) #Stop frequency
time.sleep(0.1)
gpibObj.command('SNPS2,'+str(options.numOfPoints)) #Number of points
time.sleep(0.1)
gpibObj.command('SRPT2,0') #Single shot mode
time.sleep(0.1)
if options.sweepType == 'Linear':
    sweepTypeID='0'
else:
    sweepTypeID='1'
gpibObj.command('SSTY2,'+sweepTypeID) # Sweep Type
time.sleep(0.1)
gpibObj.command('SSAM'+options.excAmp) #Source Amplitude
time.sleep(0.1)


#Start measurement
print 'Transfer function measurement started:',
sys.stdout.flush()
numPoints=int(gpibObj.query('SNPS?0')) #Number of points
time.sleep(0.1)
gpibObj.command('STRT') #Source Amplitude
time.sleep(0.1)

#Wait for the measurement to end
measuring = True
percentage=0
accomplished=0
progressInfo=termstatus.statusTxt('0%')
while measuring:
    #Get status 
    measuring = not int(gpibObj.query('DSPS?4'))
    a=int(gpibObj.query('SSFR?'))
    percentage=int(math.floor(100*a/numPoints))
    progressInfo.update(str(percentage)+'%')
    accomplished=percentage
    time.sleep(0.3)

        
progressInfo.end('100%')
gpibObj.command('ASCL1')
time.sleep(0.1)
gpibObj.command('ASCL2')
time.sleep(0.1)

#Download Data
time.sleep(2)

#Apply calibration trace to get just difference of data.
if options.useCal:
    print 'Applying calibration measurement...'
    #Set SR785's SS user function 1 to subtract calibration trace from this data.
    #gpibObj.command('USRS1,-101,75,-7,-5,-101,2,-7')
    gpibObj.command('USRS1,75,-5,2')
    time.sleep(0.1)
    #gpibObj.command('USRS2,-114,75,-7,-2,-114,2,-7')
    gpibObj.command('USRS2,75,-5,2')
    #Put user function on both displays, with calibration measurement subtracted.
    gpibObj.command('MEAS0,48')
    time.sleep(0.1)
    gpibObj.command('MEAS1,49')
    time.sleep(0.1)
    gpibObj.command('VIEW0,0') # Disp 0 = LogMag
    time.sleep(0.1)
    gpibObj.command('UNDB0,1') # dB ON
    time.sleep(0.1)
    gpibObj.command('VIEW1,6') # Disp 1 = Unwrapped Phase (std Phase ='5')
    time.sleep(0.1)

data=[]
for disp in range(2):
    print('Downloading data from display #'+str(disp))
    (f,d)=SR785.downloadData(gpibObj, disp)
    f.pop()
    d.pop()
    #Account for SR785's inexplicable use of power dB instead of voltage dB
    #when a user function is used by just doubling the 
    if ((options.useCal) and (disp==0)):
        temp = []
        for i in d:
            temp.append(str(float(i)*2))
        d = temp
    data.append(d)
    
freq=f

# Save trace pre-calibration as new calibration trace.
if options.saveCal:
    gpibObj.command('MEAS2,47')
    time.sleep(3)
    gpibObj.command('SVTR0,2')

# Apply correction factor to magnitude data
# Convert dB data to float
floatdata = [float(item) for item in data[0]]
# Correct dB data using options
caldata = map((lambda x: x-options.magcorrection), floatdata)
# Cast float data back to string
strcaldata = map((lambda x: "%e" % x), caldata)

# Data indexes for 0.1Hz to 10kHz
# Conditional statements added here and later to allow for
# measurements that don't cover the full range.
# DEA 14 Apr 2011
if ('+1.0000000e-001' in freq):
    FI1=freq.index('+1.0000000e-001')
if ('+1.0000000e+000' in freq):
    FI2=freq.index('+1.0000000e+000')
if ('+1.0000000e+001' in freq):
    FI3=freq.index('+1.0000000e+001')
if ('+1.0000000e+002' in freq):
    FI4=freq.index('+1.0000000e+002')
if ('+1.0000000e+003' in freq):
    FI5=freq.index('+1.0000000e+003')
if ('+1.0000000e+004' in freq):
    FI6=freq.index('+1.0000000e+004')

magCorrString=str(options.magcorrection)

# Apply correction factor to phase data
# Convert dB data to float
floatphasedata = [float(item) for item in data[1]]
# Correct dB data using options
calphasedata = map((lambda x: x-options.phasecorrection), floatphasedata)
# Cast float data back to string
strcalphasedata = map((lambda x: "%e" % x), calphasedata)

phaseCorrString=str(options.phasecorrection)




# Print a summary table to fill in the test document
print '----- Transfer Function Test Summary -----'
print 'Calibrated Mag(dB) = Mag(dB) - '+magCorrString+'(dB)'
print
print 'Freq(Hz)\tMag(dB)\t\tCalibratedMag(dB)'
if ('+1.0000000e-001' in freq):
    print str(float(freq[FI1]))+'\t\t'+str(float(data[0][FI1]))+'\t'+str(caldata[FI1])
if ('+1.0000000e+000' in freq):
    print str(float(freq[FI2]))+'\t\t'+str(float(data[0][FI2]))+'\t'+str(caldata[FI2])
if ('+1.0000000e+001' in freq):
    print str(float(freq[FI3]))+'\t\t'+str(float(data[0][FI3]))+'\t'+str(caldata[FI3])
if ('+1.0000000e+002' in freq):
    print str(float(freq[FI4]))+'\t\t'+str(float(data[0][FI4]))+'\t'+str(caldata[FI4])
if ('+1.0000000e+003' in freq):
    print str(float(freq[FI5]))+'\t\t'+str(float(data[0][FI5]))+'\t'+str(caldata[FI5])
if ('+1.0000000e+004' in freq):
    print str(float(freq[FI6]))+'\t\t'+str(float(data[0][FI6]))+'\t'+str(caldata[FI6])
print
print 'Calibrated Phase(deg) = Phase(deg) - '+phaseCorrString+'(deg)'
print
print 'Freq(Hz)\tPhase(deg)\tCalibratedPhase(deg)'
if ('+1.0000000e-001' in freq):
    print str(float(freq[FI1]))+'\t\t'+str(float(data[1][FI1]))+'\t'+str(calphasedata[FI1])
if ('+1.0000000e+000' in freq):
    print str(float(freq[FI2]))+'\t\t'+str(float(data[1][FI2]))+'\t'+str(calphasedata[FI2])
if ('+1.0000000e+001' in freq):
    print str(float(freq[FI3]))+'\t\t'+str(float(data[1][FI3]))+'\t'+str(calphasedata[FI3])
if ('+1.0000000e+002' in freq):
    print str(float(freq[FI4]))+'\t\t'+str(float(data[1][FI4]))+'\t'+str(calphasedata[FI4])
if ('+1.0000000e+003' in freq):
    print str(float(freq[FI5]))+'\t\t'+str(float(data[1][FI5]))+'\t'+str(calphasedata[FI5])
if ('+1.0000000e+004' in freq):
    print str(float(freq[FI6]))+'\t\t'+str(float(data[1][FI6]))+'\t'+str(calphasedata[FI6])
print '----------'

#Open file
dataFile = open(dataFileName,'w')

#Write to the data file
print('Writing measurement data to file...')

#Deal with an empty title
if options.title == "":
    options.title = options.filename

#Write parameters to file
dataFile.write('# SR785 SUS Transfer Function Measurement\n')
dataFile.write('Title: '+options.title+'\n')
dataFile.write('Memo: '+options.memo+'\n')
dataFile.write('Instrument ID: '+idnString+'\n')
dataFile.write('#---------- Measurement Setup ------------\n')
dataFile.write('Start frequency = '+options.startFreq+'\n')
dataFile.write('Stop frequency = '+options.stopFreq+'\n')
dataFile.write('Number of frequency points = '+str(options.numOfPoints)+'\n')
dataFile.write('Excitation amplitude = '+options.excAmp+'\n')
dataFile.write('Settling cycles = '+str(options.settleCycles)+'\n')
dataFile.write('Integration cycles = '+str(options.intCycles)+'\n')
dataFile.write('\n')

SR785.getparam(gpibObj, options.filename, dataFile)

dataFile.write('#---------- Transfer Function Test Summary -----------\n')
dataFile.write('Calibrated Mag(dB) = Mag(dB) - '+magCorrString+'(dB)\n')
dataFile.write('Calibrated Phase(deg) = Phase(deg) - '+phaseCorrString+'(deg)\n')
dataFile.write('[Frequency (Hz) Mag(dB) CalibratedMag(db) Phase(deg) CalibratedPhase(deg)]\n')
if ('+1.0000000e-001' in freq):
    dataFile.write(freq[FI1]+' '+data[0][FI1]+' '+strcaldata[FI1]+' '+data[1][FI1]+' '+strcalphasedata[FI1]+'\n')
if ('+1.0000000e+000' in freq):
    dataFile.write(freq[FI2]+' '+data[0][FI2]+' '+strcaldata[FI2]+' '+data[1][FI2]+' '+strcalphasedata[FI2]+'\n')
if ('+1.0000000e+001' in freq):
    dataFile.write(freq[FI3]+' '+data[0][FI3]+' '+strcaldata[FI3]+' '+data[1][FI3]+' '+strcalphasedata[FI3]+'\n')
if ('+1.0000000e+002' in freq):
    dataFile.write(freq[FI4]+' '+data[0][FI4]+' '+strcaldata[FI4]+' '+data[1][FI4]+' '+strcalphasedata[FI4]+'\n')
if ('+1.0000000e+003' in freq):
    dataFile.write(freq[FI5]+' '+data[0][FI5]+' '+strcaldata[FI5]+' '+data[1][FI5]+' '+strcalphasedata[FI5]+'\n')
if ('+1.0000000e+004' in freq):
    dataFile.write(freq[FI6]+' '+data[0][FI6]+' '+strcaldata[FI6]+' '+data[1][FI6]+' '+strcalphasedata[FI6]+'\n')
dataFile.write('\n')

#Curve-fitting code. I don't recommend using it, the SR785 simply
#isn't very good at fitting curves.
if options.curvefit:
    time.sleep(1)
    #Set SR785's SS user function 1 to subtract calibration trace from this data.
    gpibObj.command("USRS1,75,-2,2")
    time.sleep(0.1)
    #Put user function on display 0, so we can fit to it.
    gpibObj.command("MEAS0,48")
    time.sleep(0.1)
    #Set fit region or it throws up.
    gpibObj.command("ERNG0,0,"+str(options.numOfPoints-1))
    gpibObj.command("ENPL1")
    gpibObj.command("ENZE1")
    gpibObj.command("EPOL0,0 -1000")
    gpibObj.command("EZER0,0 -20000")
    time.sleep(1)
    print 'Fitting 1 pole and 1 zero...'
    gpibObj.command("EFIT0")
    time.sleep(4)
    pole = gpibObj.query("EPOL?0,0")
    zero = gpibObj.query("EZER?0,0")
    print 'Pole: '+str(pole)+' Zero: '+str(zero)
    dataFile.write('\n#---------- Curve Fit Results (Iffy) -----------\n')
    dataFile.write('Pole: '+str(pole)+' Zero: '+str(zero)+'\n')
    dataFile.write('\n')

dataFile.write('#---------- Instrument Data -----------\n')
dataFile.write('# Column format = [Freq  Mag(dB) CalibratedMag(dB) Phase(deg) CalibratedPhase(deg)]\n')

#Write data to file
for i in range(len(freq)):
    dataFile.write(freq[i])
    dataFile.write(' '+data[0][i])
    dataFile.write(' '+strcaldata[i])
    dataFile.write(' '+data[1][i])
    dataFile.write(' '+strcalphasedata[i])
    dataFile.write('\n')

dataFile.close()

#Plot results for extra fanciness.
print 'Plotting data...'
fig = plt.figure()
fig.suptitle(str(dataFileName))
magplot = fig.add_subplot(2,1,1)
magplot.semilogx(freq,data[0])
magplot.set_xlabel('Frequency (Hz)')
magplot.set_ylabel('Magnitude (dB)')
phaseplot = fig.add_subplot(2,1,2)
phaseplot.semilogx(freq,data[1])
phaseplot.set_xlabel('Frequency (Hz)')
phaseplot.set_ylabel('Phase (deg)')
plt.show()

print "done."
gpibObj.close()

